/*
 * Synchronization for bounded-buffer problem, using locks and 
 * condition variables to simulate monitors.
 */
#include <pthread.h>
#include "bbuffer.h"

/* global variables */

pthread_mutex_t monitor_mutex; 
pthread_cond_t not_empty;
pthread_cond_t not_full;

/* function definitions */

void put_with_synch(const int producerID, const bbuffer_item_t item, bbuffer_t * buffer) {
    pthread_mutex_lock(&monitor_mutex);
    /* 
     * "while" not "if" because POSIX implementation of condition
     * variables sometimes generates spurious signals.
     */
    while (buffer->count == buffer->size) {
        pthread_cond_wait(&not_full, &monitor_mutex);
    }
    bbuffer_insert(&buffer, item);
    printf("at time %ld producer %d added item, count now %d\n", 
            elapsed_time(), producerID, buffer->count);
    pthread_cond_signal(&not_empty);
    pthread_mutex_unlock(&monitor_mutex);
}

bbuffer_item_t get_with_synch(const int consumerID, bbuffer_t * buffer) {
    pthread_mutex_lock(&monitor_mutex);
    /* 
     * "while" not "if" because POSIX implementation of condition
     * variables sometimes generates spurious signals.
     */
    while (buffer->count == 0) {
        pthread_cond_wait(&not_empty, &monitor_mutex);
    }
    bbuffer_item_t item = bbuffer_remove(&buffer);
    printf("at time %ld consumer %d removed item, count now %d\n", 
            elapsed_time(), consumerID, buffer->count);
    pthread_cond_signal(&not_full);
    pthread_mutex_unlock(&monitor_mutex);
    return item;
}
