/*
 * Synchronization for bounded-buffer problem, using semaphores.
 */
#include <pthread.h>
#include <semaphore.h>
#include "bbuffer.h"

/* global variables */

sem_t buffer_mutex;
sem_t empty;
sem_t full;

/* function definitions */

void put_with_synch(const int producerID, const bbuffer_item_t item, bbuffer_t * buffer) {
    sem_wait(&empty);
    sem_wait(&buffer_mutex);
    bbuffer_insert(buffer, item);
    printf("at time %ld producer %d added item, count now %d\n", 
                elapsed_time(), producerID, buffer->count);
    sem_post(&buffer_mutex);
    sem_post(&full);
}

bbuffer_item_t get_with_synch(const int consumerID, bbuffer_t * buffer) {
    sem_wait(&full);
    sem_wait(&buffer_mutex);
    bbuffer_item_t item = bbuffer_remove(buffer);
    printf("at time %ld consumer %d removed item, count now %d\n", 
            elapsed_time(), consumerID, buffer->count);
    sem_post(&buffer_mutex);
    sem_post(&empty);
    return item;
}
